﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms.DataVisualization.Charting;

//波浪类库：二维行进波
//袁绪龙，2023-9-3
//为《海洋工程波浪力学》教学演示系统设计
namespace ClassLibraryWave
{
    /// <summary>
    /// 二维行进波
    /// </summary>
    /// <remarks>从基类Wave派生</remarks>
    public class MarchingWave :Wave
    { 
        /// <summary>
        /// 设置波浪参数
        /// </summary>
        /// <param name="waveH">波高</param>
        /// <param name="waveL">波长</param>
        /// <param name="waveT">周期</param>
        /// <param name="waveDepth">水深</param>
        /// <param name="nPoint">波形坐标点数</param>
        /// <param name="nWave">计算波长数</param>
        public override void Set(double waveH,double waveL,double waveT,double waveDepth,int nPoint=40,int nWave=2)
        {
            H = waveH;
            L = waveL;
            T = waveT;
            d = waveDepth;
            a = 0.5 * H;
            k = 2 * Math.PI / L;
            w = 2 * Math.PI / T;
            t = 0;
            this.NPoint = nPoint;
            this.NWave = nWave;
            x = new double[nPoint * nWave + 1];
            z = new double[nPoint * nWave + 1];
        }

        /// <summary>
        /// 计算波形数组
        /// </summary>
        /// <param name="nPoint">每周期点数</param>
        /// <param name="nWave">波的个数</param>
        /// <param name="t">当前时刻</param>
        public override void Calc(double t)
        {
            this.t = t;
            double dx = L / NPoint;
            for (int i = 0; i < NPoint * NWave + 1; i++)
            {
                x[i] = i * dx;
                z[i] = a * Math.Cos(k * x[i] - w * t);
            }
        }

        /// <summary>
        /// 显示波面曲线
        /// </summary>
        /// <param name="chart1">图表控件</param>
        public override void Show(Chart chart1)
        {
            chart1.Series[0].Points.Clear();
            for (int i = 0; i < NPoint * NWave + 1; i++)
            {
                chart1.Series[0].Points.AddXY(x[i], z[i]);
            }
        }


        /// <summary>
        /// 设置轨迹参数
        /// </summary>
        /// <remarks>抽象方法，用于定义统一接口</remarks>
        /// <param name="x0">质点位置</param>
        /// <param name="nTraj">质点数量</param>
        public override void SetTraj(double x0=0.25, int nTraj=5)
        {
            this.x0 = x0*L;
            this.NTraj = nTraj;
            z0 = new double[nTraj];
            xt = new double[nTraj];
            zt = new double[nTraj];
            for (int i = 0; i < nTraj; i++)
            {
                z0[i] = -i * d / (nTraj-1);
            }
        }

        /// <summary>
        /// 计算轨迹坐标
        /// </summary>
        /// <remarks>抽象方法，用于定义统一接口</remarks>
        /// <param name="t">当前时间</param>
        public override void CalcTraj(double t)
        {
            for (int i = 0; i < NTraj; i++)
            {
                double t1=Math.Cosh(k*(z0[i]+d));
                double t2=Math.Sinh(k*(z0[i]+d));
                double t3=Math.Sinh(k*d);
                xt[i]=x0-a*t1/t3*Math.Sin(k*x0-w*t);
                zt[i] = z0[i] + a * t2 / t3 * Math.Cos(k * x0 - w * t);
            }
        }

        /// <summary>
        /// 显示水质点轨迹曲线
        /// </summary>
        /// <remarks>抽象方法，用于定义统一接口</remarks>
        /// <param name="chart1">图表控件</param>
        public override void ShowTraj(Chart chart1)
        {
            for (int i = 0; i < NTraj; i++)
            {
                chart1.Series[i + 1].Points.AddXY(xt[i], zt[i]);
                if (chart1.Series[i + 1].Points.Count >= T * 10) chart1.Series[i + 1].Points.Clear();
            }
        }
    }
}
